import 'mocha';
import assert from 'assert';
import { compile } from '../parserlib.js';
import { visualizeAsUrl, visualizeAsHtml } from '../internal/visualizer.js';
describe('visualizer', () => {
    it('should handle Lec17i example', () => {
        let URL2;
        (function (URL2) {
            URL2[URL2["URL"] = 0] = "URL";
            URL2[URL2["HOSTNAME"] = 1] = "HOSTNAME";
            URL2[URL2["PORT"] = 2] = "PORT";
            URL2[URL2["WORD"] = 3] = "WORD";
        })(URL2 || (URL2 = {}));
        ;
        const grammar = "url ::= 'http://' hostname (':' port)? '/';" +
            "hostname ::= word '.' hostname | word '.' word;" +
            "port ::= [0-9]+;" +
            "word ::= [a-z]+;";
        const parser = compile(grammar, URL2, URL2.URL);
        const tree = parser.parse("http://mit.google.com:123/");
        const url = visualizeAsUrl(tree, URL2);
        //console.error(url);
        assert(url.includes("nd%28%22url%22%2Cnd%28%22%27http%3A%2F%2F%27%22%2C%20uu%2C%20nd%28%22hostname%22%2Cnd%28%22word%22%2Cnd%28%22%27mit%27%22%29%2Cnd%28%22%27.%27%22%2C%20uu%2C%20nd%28%22hostname%22%2Cnd%28%22word%22%2Cnd%28%22%27google%27%22%29%2Cnd%28%22%27.%27%22%2C%20uu%2C%20nd%28%22word%22%2Cnd%28%22%27com%27%22%29%2Cuu%29%29%29%2Cuu%29%29%29%2Cnd%28%22%27%3A%27%22%2C%20uu%2C%20nd%28%22port%22%2Cnd%28%22%27123%27%22%29%2Cnd%28%22%27%2F%27%22%2C%20uu%2C%20uu%29%29%29%29%29%2Cuu%29"));
    });
    it('should gray out skipped subtrees', () => {
        let J17;
        (function (J17) {
            J17[J17["ROOT"] = 0] = "ROOT";
            J17[J17["SUM"] = 1] = "SUM";
            J17[J17["WHITESPACE"] = 2] = "WHITESPACE";
            J17[J17["PRIMITIVE"] = 3] = "PRIMITIVE";
            J17[J17["NUMBER"] = 4] = "NUMBER";
        })(J17 || (J17 = {}));
        ;
        const grammar = "root ::= sum; "
            + "@skip whitespace{"
            + "sum ::= primitive ('+' primitive)*;"
            + "primitive ::= number | '(' sum ')';"
            + "} "
            + "whitespace ::= [ \\t\\r\\n]+;"
            + "number ::= [0-9]+;";
        const parser = compile(grammar, J17, J17.ROOT);
        const tree = parser.parse("1 +\t 2 + \n 3");
        const url = visualizeAsUrl(tree, J17);
        //console.error(url);
        assert(url.includes("nd%28%22root%22%2Cnd%28%22sum%22%2Cnd%28%22primitive%22%2Cnd%28%22number%22%2Cnd%28%22%271%27%22%29%2Cnd%28%22whitespace%22%2Cnd%28%22%27%20%27%22%29%2Cuu%2Ctrue%29%29%2Cnd%28%22%27%2B%27%22%2C%20uu%2C%20nd%28%22whitespace%22%2Cnd%28%22%27%09%20%27%22%29%2Cnd%28%22primitive%22%2Cnd%28%22number%22%2Cnd%28%22%272%27%22%29%2Cnd%28%22whitespace%22%2Cnd%28%22%27%20%27%22%29%2Cuu%2Ctrue%29%29%2Cnd%28%22%27%2B%27%22%2C%20uu%2C%20nd%28%22whitespace%22%2Cnd%28%22%27%20%5Cn%20%27%22%29%2Cnd%28%22primitive%22%2Cnd%28%22number%22%2Cnd%28%22%273%27%22%29%2Cuu%29%2Cuu%29%2Ctrue%29%29%29%2Ctrue%29%29%29%2Cuu%29%2Cuu%29"));
    });
    it('should handle large trees', () => {
        let S;
        (function (S) {
            S[S["S"] = 0] = "S";
            S[S["N"] = 1] = "N";
        })(S || (S = {}));
        ;
        const grammar = "s ::= n+;  n ::= \\d;";
        const parser = compile(grammar, S, S.S);
        const numZeros = 500;
        const input = "".padStart(numZeros, '0');
        const tree = parser.parse(input);
        const url = visualizeAsUrl(tree, S);
        const html = visualizeAsHtml(tree, S);
        const expectedCode = "nd%28%22s%22%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cnd%28%22n%22%2Cnd%28%22%270%27%22%29%2Cuu%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%29%2Cuu%29";
        //console.error(html);
        assert(url.includes(expectedCode));
        assert(html.includes(expectedCode));
    });
    it('should handle quotes and linebreaks in terminals', () => {
        let Grammar;
        (function (Grammar) {
            Grammar[Grammar["Root"] = 0] = "Root";
        })(Grammar || (Grammar = {}));
        ;
        const grammar = "root ::= .*;";
        const parser = compile(grammar, Grammar, Grammar.Root);
        const tree = parser.parse("some \"double-quoted\" and \'single-quoted\' and \\backslashes\\ and \nnewline\n and \r\nWindows newline\r\n");
        const url = visualizeAsUrl(tree, Grammar);
        //console.error(url);
        assert(url.includes("nd%28%22root%22%2Cnd%28%22%27some%20%5C%22double-quoted%5C%22%20and%20%27single-quoted%27%20and%20%5C%5Cbackslashes%5C%5C%20and%20%5Cnnewline%5Cn%20and%20%5Cr%5CnWindows%20newline%5Cr%5Cn%27%22%29%2Cuu%29"));
    });
});
//# sourceMappingURL=visualizer-test.js.map